from collections.abc import Iterator
from typing import Any, Optional

from oryks_google_oauth import GoogleOAuth, YouTubeScopes
from pydantic import BaseModel

from .models import (
    Caption,
    Channel,
    ChannelSection,
    Comment,
    CommentThread,
    Playlist,
    PlaylistItem,
    Subscription,
    Video,
    VideoCategory,
)
from .resources import YouTubeSearch
from .schemas import (
    CreatePlaylist,
    CreatePlaylistItem,
    InsertChannelSection,
    LanguageResponse,
    RegionResponse,
    ThumbnailSetResponse,
    UploadVideo,
    VideoReportAbuse,
    VideoReportReasonResponse,
    YouTubeListResponse,
    YouTubeRequest,
    YouTubeResponse,
)


class YouTube(BaseModel):
    """Provides methods for interacting with the YouTube API.

    This class acts as an interface to the YouTube API, providing methods for interacting with
    the YouTube V3 API.

    Attributes
    ----------
    client_secret_file: str
        The path to the json file containing your authentication information.
    """

    client_secret_file: Optional[str] = None
    authenticated: Optional[bool] = False
    youtube_client: Optional[Any] = None

    def authenticate(self, client_secret_file: Optional[str] = None) -> None:
        """Authenticate the requests made to youtube.

        Used to generate the credentials that are used when authenticating requests to youtube.

        Parameters
        ----------
        client_secret_file: str
            The path to clients secret json file from Google

        Raises
        ------
        ValueError:
            When the client secrets file is not provided
        FileNotFoundError:
            When the secrets file path is not found
        """
        if client_secret_file:
            self.client_secret_file = client_secret_file
        if not self.client_secret_file:
            raise ValueError('The client secret file must be provided.')
        api_service_name: str = 'youtube'
        api_version: str = 'v3'
        credentials_dir: str = '.youtube'
        scopes: list[str] = [YouTubeScopes.youtube.value]
        oauth: GoogleOAuth = GoogleOAuth(
            secrets_file=self.client_secret_file,
            scopes=scopes,
            api_service_name=api_service_name,
            api_version=api_version,
            credentials_dir=credentials_dir,
        )
        self.youtube_client = oauth.authenticate_google_server()

    def search(self, search_schema: YouTubeRequest) -> YouTubeResponse:
        """Used to search through youtube for videos, playlists and channels.

        Parameters
        ----------
        search_schema: YouTubeRequest
            An instance of YouTubeRequest that contains all the details needed to search.
        Returns
        -------
        YouTubeResponse:
            An instance of YouTubeResponse that has al the search results.
        """
        search: YouTubeSearch = YouTubeSearch(youtube_client=self.youtube_client)
        search_results: YouTubeResponse = search.search(search_schema)
        return search_results

    def get_search_iterator(self, search_schema: YouTubeRequest) -> Iterator:
        """Return an iterator that can be used to iterate over search results.

        Parameters
        ----------
        search_schema: YouTubeRequest
            An instance of YouTubeRequest that contains all the details needed to search.
        Returns
        -------
        Iterator:
            An iterator that can be used to iterate over the search results.
        """
        search: YouTubeSearch = YouTubeSearch(youtube_client=self.youtube_client)
        return search.get_search_iterator(search_schema)

    def find_channel_by_name(self, display_name: str) -> YouTubeResponse:
        """Find a channel's details when given the channel title.

        Parameters
        ----------
        display_name: str
            The name of the youtube channel.
        Returns
        -------
        YouTubeResponse:
            A youtube response consisting of channels whose names match the provided names
        """
        search: YouTubeSearch = YouTubeSearch(youtube_client=self.youtube_client)
        return search.find_channel_by_name(display_name)

    def get_video_ratings(self, video_ids: list[str]) -> list[str]:
        """Find out whether or not you liked the videos whose ids are provided.

        You provide it with a list of video ids and for each video it will tell you whether
        or not you have liked that video.

        Parameters
        ----------
        video_ids: list[str]:
            A list of strings of video ids
        Returns
        -------
        list[str]:
            A list of strings showing whether or not you liked a given video.
        """
        pass

    def find_video_by_id(self, video_id: str) -> Video:
        """Find a single video by providing the video's id.

        Parameters
        ----------
        video_id: str
            The video's id
        Returns
        -------
        Video:
            A Video instance
        """
        pass

    def find_videos(self, video_ids: list[str]) -> list[Video]:
        """Find a many videos by providing a list of video ids.

        Parameters
        ----------
        video_ids: list[str]
            A list of video ids
        Returns
        -------
        list[Video]:
            A list of Video instances
        """
        pass

    def find_most_popular_video_by_region(
        self, region_code: str = 'US', category_id: str = ''
    ) -> list[Video]:
        """Get the most popular videos in a given region and category.

        Parameters
        ----------
        region_code: str
            A string representing the region code.
        category_id: str
            The category to find. This is a string number i.e '22'
        Returns
        -------
        list[Video]:
            A list of the most popular videos
        """
        pass

    def report_video_abuse(self, abuse_report: VideoReportAbuse) -> None:
        """Report a video that you deem as harmful or infringing on copyrights.

        Parameters
        ----------
        abuse_report: VideoReportAbuse
            An instance of VideoReportAbuse with all the details needed to report.
        """
        pass

    def update_video(self, upload_req: UploadVideo) -> Video:
        """Updates a video's metadata."""
        raise NotImplementedError()

    def delete_video(self, video_id: str) -> None:
        """Deletes a YouTube video."""
        raise NotImplementedError()

    def upload_video(self, upload_req: UploadVideo) -> Video:
        """Upload a YouTube video."""
        raise NotImplementedError()

    def rate_video(self, video_id: str, rating: str) -> None:
        """Add a like or dislike rating to a video or remove a rating from a video."""
        raise NotImplementedError()

    def find_channel_by_id(self, channel_id: str) -> Channel:
        """Find a youtube channel given its id."""
        raise NotImplementedError()

    def find_channels_by_ids(self, channel_ids: list[str]) -> list[Channel]:
        """Find many channels given their ids."""
        raise NotImplementedError()

    def find_my_channel(self) -> Channel:
        """Find the details fo your youtube channel."""
        raise NotImplementedError()

    def find_channel_playlists(
        self, channel_id: str, max_results: int = 10
    ) -> YouTubeResponse:
        """Find the playlists for a given channel."""
        raise NotImplementedError()

    def find_my_playlists(self) -> YouTubeResponse:
        """Find the playlists in your channel."""
        raise NotImplementedError()

    def get_my_playlists_iterator(self, max_results: Optional[int] = 10) -> Iterator:
        """Get an iterator for iterating through playlists in your channel."""
        raise NotImplementedError()

    def insert_playlist(self, playlist_schema: CreatePlaylist) -> Playlist:
        """Create a new playlist in your channel."""
        raise NotImplementedError()

    def update_playlist(
        self, playlist_id: str, playlist_schema: CreatePlaylist
    ) -> Playlist:
        """Update a playlist in your channel."""
        raise NotImplementedError()

    def delete_playlist(self, playlist_id: str) -> None:
        """Delete a playlist in your channel."""
        raise NotImplementedError()

    def find_playlist_items(
        self, playlist_id: str, max_results: Optional[int] = 25
    ) -> YouTubeResponse:
        """Find a particular video in your playlist."""
        raise NotImplementedError()

    def find_playlist_items_by_ids(
        self, playlist_item_ids: list[str]
    ) -> YouTubeResponse:
        """Find various playlist videos given their ids."""
        raise NotImplementedError()

    def insert_playlist_item(self, create_item: CreatePlaylistItem) -> PlaylistItem:
        """Add a video to a playlist."""
        raise NotImplementedError()

    def update_playlist_item(
        self, playlist_id: str, playlist_item_id: str, video_id: str, position: int = 1
    ) -> PlaylistItem:
        """Update a given video details in a playlist."""
        raise NotImplementedError()

    def delete_playlist_item(self, playlist_item_id: str) -> Playlist:
        """Delete a video from a playlist."""
        raise NotImplementedError()

    def get_video_categories(
        self, region_code: Optional[str] = ''
    ) -> list[VideoCategory]:
        """List all the video categories on youtube."""
        raise NotImplementedError()

    def find_video_comments(self, request: YouTubeRequest) -> YouTubeResponse:
        """Get a particular video's comments."""
        raise NotImplementedError()

    def get_comments_iterator(self, request_schema: YouTubeResponse) -> Iterator:
        """Get an iterator for going through a videos comments."""
        raise NotImplementedError()

    def find_all_channel_comments(self, request: YouTubeRequest) -> YouTubeResponse:
        """Get a particular channels's comments."""
        raise NotImplementedError()

    def insert_comment(self, video_id: str, comment: str) -> CommentThread:
        """Comment on a given video."""
        raise NotImplementedError()

    def get_comment_replies(self, comment_id: str) -> list[Comment]:
        """Get the replies for a given comment."""
        raise NotImplementedError()

    def get_comment(self, comment_id: str) -> Comment:
        """Get a comments, given its id."""
        raise NotImplementedError()

    def get_comments(self, comment_ids: list[str]) -> list[Comment]:
        """Get various comments given their ids."""
        raise NotImplementedError()

    def reply_to_comment(self, comment_id: str, comment: str) -> Comment:
        """Reply to the given comment."""
        raise NotImplementedError()

    def update_comment(self, comment_id: str, comment: str) -> Comment:
        """Update a comments."""
        raise NotImplementedError()

    def delete_comment(self, comment_id: str) -> None:
        """Delete a comment."""
        raise NotImplementedError()

    def list_channel_activity(self, request: YouTubeRequest) -> YouTubeResponse:
        """List all the activities of the given channel, such as uploads."""
        raise NotImplementedError()

    def list_my_activities(self, request: YouTubeRequest) -> YouTubeResponse:
        """List all the activities for your channel, such as creating a playlist."""
        raise NotImplementedError()

    def list_video_captions(self, video_id: str) -> Caption:
        raise NotImplementedError()

    def insert_video_caption(self, video_id: str) -> None:
        raise NotImplementedError()

    def update_video_caption(self, caption_id: str) -> None:
        raise NotImplementedError()

    def download_video_caption(self, caption_id: str) -> None:
        raise NotImplementedError()

    def delete_video_caption(self, caption_id: str) -> None:
        raise NotImplementedError()

    def upload_channel_banner(self, media_file: str, channel_id: str) -> None:
        raise NotImplementedError()

    def list_channel_sections(self, channel_id: str) -> YouTubeListResponse:
        """List a channel's sections."""
        raise NotImplementedError()

    def list_my_channel_sections(self) -> YouTubeListResponse:
        """List your channel sections."""
        raise NotImplementedError()

    def insert_channel_section(self, request: InsertChannelSection) -> ChannelSection:
        """Insert a new channle section."""
        raise NotImplementedError()

    def update_channel_section(self, channel_section_id: str) -> None:
        """Update a channel section."""
        raise NotImplementedError()

    def delete_channel_section(self, channel_section_id: str) -> None:
        """Delete a channel section."""
        raise NotImplementedError()

    def list_languages(self, language: str = 'en_US') -> LanguageResponse:
        """List all the languages youtube supports."""
        raise NotImplementedError()

    def list_regions(self, language: str = 'en_US') -> RegionResponse:
        """List all the regions youtube supports."""
        raise NotImplementedError()

    def set_watermark(self, channel_id: str) -> None:
        """Set a video's watermarks."""
        raise NotImplementedError()

    def unset_watermark(self, channel_id: str) -> None:
        """Remove a video's watermark."""
        raise NotImplementedError()

    def list_video_abuse_report_reasons(self) -> VideoReportReasonResponse:
        """List reasons that can be used to report a video."""
        raise NotImplementedError()

    def set_video_thumbnail(
        self, video_id: str, thumbnail: str
    ) -> ThumbnailSetResponse:
        """Set a video's thumbnail."""
        raise NotImplementedError()

    def list_channel_subscriptions(self, request: YouTubeRequest) -> YouTubeResponse:
        """List the given channel's subscriptions."""
        raise NotImplementedError()

    def list_my_subscriptions(self, request: YouTubeRequest) -> YouTubeResponse:
        """List the channels I subscribe to."""
        raise NotImplementedError()

    def do_i_subscribe(self, channel_id: str) -> bool:
        """Check if I subscribe to the given channel."""
        raise NotImplementedError()

    def is_subscribed(self, channel_id: str, user_channel: str) -> bool:
        """Check if one channel subscribes to another channel.

        Parameters
        ----------
        channel_id: str
            The channel id of the channel to which subscription is being confirmed.
        user_channel: str
            The channel id of the channel to check if its subscribed.
        Returns
        -------
        bool:
            Whether user_channel subscribes to channel_id
        """
        raise NotImplementedError()

    def subscribe_to_channel(self, channel_id: str) -> Subscription:
        """Subscribe to a youtube channel.

        Parameters
        ----------
        channel_id: str
            The id of the channel you want to subscribe to.
        Returns
        -------
        Subscription:
            An instance of subscription with all the details on the channel subscription.
        """
        raise NotImplementedError()

    def unsubscribe_to_channel(self, subscription_id: str) -> None:
        """Unsubscribe from a youtube channel.

        Parameters
        ----------
        subscription_id: str
            The subscription id. You can get this by listing your subscriptions.
        """
        raise NotImplementedError()

    def list_activities(self) -> dict:
        """List your activities on youtube including subscriptions, likes."""
        raise NotImplementedError()
